--    cr2hdr developped by a1ex
--    http://www.magiclantern.fm/forum/index.php?topic=7139.0
--    Plugin developped for Magic Lantern
--    Add "cr2hdr" to export
--
--    Copyright (C) <2014>  <kichetof kichetof@gmail.com>
--
--    This program is free software; you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation; either version 2 of the License, or
--    (at your option) any later version.
--
--    This program is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License along
--    with this program; if not, write to the Free Software Foundation, Inc.,
--    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

local LrApplication = import 'LrApplication'
local LrBinding = import 'LrBinding'
local LrColor = import 'LrColor'
local LrDialogs = import 'LrDialogs'
local LrErrors = import 'LrErrors'
local LrExportSession = import 'LrExportSession'
local LrFileUtils = import 'LrFileUtils'
local LrFunctionContext = import 'LrFunctionContext'
local LrHttp = import 'LrHttp'
local LrLogger = import 'LrLogger'
local LrPathUtils = import 'LrPathUtils'
local LrProgressScope = import 'LrProgressScope'
local LrTasks = import 'LrTasks'
local LrView = import "LrView"

local bind = LrView.bind
local mlLog = LrLogger('MLDualISO')
local next = next

function checkDependencies()
	return "exec which dcraw exiftool"
end

mlLog:enable("print")
--mlLog:disable()

exportServiceProvider = {}

exportServiceProvider.canExportVideo = false

exportServiceProvider.exportPresetFields = {
	{ key = 'suffix',           	default = "dualiso" },
	{ key = 'label',            	default = "DualISO" },
	{ key = 'saveOutput',       	default = true },
	{ key = 'addToCollection',  	default = true },
	{ key = 'subfolder',        	default = "" },
	{ key = 'verbose',				default = false },
--	{ key = 'moveToTrash',			default = false },
	{ key = 'shortcut_fast',		default = false },
	{ key = 'interp_method',		default = 0 }, --0 amaze, 1 mean23
	{ key = 'chroma_smooth_method',	default = 2 }, --2 cs2, 3 cs3, 5 cs5, 0 no-cs
	{ key = 'fix_bad_pixels',		default = 0 }, --1 bad, 2 really bad, 0 no bad
	{ key = 'debug_bad_pixels',		default = false },
	{ key = 'use_fullres', 			default = 0 }, --0 no full, 1 full
	{ key = 'use_alias_map',		default = 0 }, --0 no map, 1 map
	{ key = 'use_stripe_fix',		default = 0 }, --0 no stripe, 1 stripe
	{ key = 'compress',				default = 0 }, --0 no compress, 1 compress, 2 compress lossy
}

exportServiceProvider.hideSections = { "exportLocation", "fileNaming", "fileSettings", "imageSettings", "outputSharpening", "metadata", "watermarking", "video" }

function exportServiceProvider.endDialog( propertyTable )

	propertyTable.LR_tokens = string.format("{{image_name}}-%s", propertyTable.suffix)
	propertyTable.LR_format = 'ORIGINAL'
	propertyTable.LR_export_colorSpace = "AdobeRGB"

end

function exportServiceProvider.processRenderedPhotos(functionContext, exportContext)

	functionContext:addCleanupHandler(
		function(status, message)
	        if not status then
	        
	              LrDialogs.message("~ There has been an error ~", string.format("Please report the problem to Magic Lantern forum\n\n%s", message))
	        
	        end
	    end
    )
	
	local exportSession = exportContext.exportSession
	local exportSettings = exportContext.propertyTable
	local nPhotos = exportSession:countRenditions()
	local progressScope = exportContext:configureProgress {
		title = nPhotos > 1 and LOC("$$$/ML/Publish/Progress=Exporting ^1 photos to cr2hdr", nPhotos) or LOC "$$$/ML/Publish/Progress/One=Exporting one photo to cr2hdr",
	}

	for i, rendition in exportContext:renditions { stopIfCanceled = true } do
	
		if MAC_ENV then
		
			local cmd = checkDependencies()
			isDependOk = LrTasks.execute(cmd)

			if isDependOk ~= 0 then
				
				LrDialogs.message(LOC "$$$/ML/Dependencies/ErrorTitle=~ Dependencies error ~", LOC "$$$/ML/Dpendencies/Error=This plugin (cr2hdr) need dcraw and exiftool.^n^nPlease install it before run this plugin")
				break
				
			end
		end
	
		local fileFormat = rendition.photo:getRawMetadata("fileFormat")
		if fileFormat == "RAW" or fileFormat == "DNG" then
			
			progressScope:setPortionComplete((i - 1) / nPhotos)
			
			if not rendition.wasSkipped then
			
				local success, pathOrMessage = rendition:waitForRender()
				progressScope:setPortionComplete((i - 0.5) / nPhotos)
				
				local originalFilePath = rendition.photo.path
				mlLog:debugf("originalFilePath %s", originalFilePath)
				-- original filename to export (exemple img_1234.cr2)
				local originalName = LrPathUtils.leafName(originalFilePath)
				mlLog:debugf("originalName %s", originalName)
				-- original path to reimport
				local originalPath = LrPathUtils.parent(originalFilePath)
				
				if exportSettings.subfolder ~= "" then
				
					originalPath = LrPathUtils.child(originalPath, exportSettings.subfolder)
					local createSubFolder = LrFileUtils.createAllDirectories(originalPath)
					mlLog:debugf("createSubFolder : %s", createSubFolder)
					
				end
				
				mlLog:debugf("originalPath %s", originalPath)
				-- get original collection
				local originalCollection = rendition.photo:getContainedCollections()
				mlLog:debugf("originalCollection %s", originalCollection)
				-- export path & filename (exemple ~/images/dualiso_1234.cr2)
				local filePath = assert(pathOrMessage)
				mlLog:debugf("filePath %s", filePath)
				-- export path & filename for DNG file
				local filePathDNG = LrPathUtils.replaceExtension(filePath, "DNG")
				mlLog:debugf("filePathDNG %s", filePathDNG)
				-- the filename for reimport to LR
				local importFileDNG = LrPathUtils.leafName(filePathDNG)
				mlLog:debugf("importFileDNG %s", importFileDNG)
				-- the filename & path to reimport into LR
				local importDNG = LrPathUtils.child(originalPath, importFileDNG)
				-- possibility to bypass same name; disable to see already treated message
				--local importDNG = LrFileUtils.chooseUniqueFileName(importDNG)
				mlLog:debugf("importDNG %s", importDNG)
				-- export path & filename for txt output command
				local output = LrPathUtils.replaceExtension(importDNG, "txt")
				mlLog:debugf("output %s", output)
				
				if LrFileUtils.exists(importDNG) == false then
				
					if progressScope:isCanceled() then break end
					
					if success then
					
						-- plugin path
						local pluginPath = _PLUGIN.path
						
						local args = string.format('"%s%s%s%s%s%s%s%s%s"', 
							(exportSettings.shortcut_fast == true) and " --fast" or "",
							(exportSettings.interp_method == 0) and " --amaze-edge" or " --mean23",
							(exportSettings.chroma_smooth_method == 0) and " --no-cs" or string.format(" --cs%dx%d", exportSettings.chroma_smooth_method, exportSettings.chroma_smooth_method),
							(exportSettings.fix_bad_pixels == 0) and " --no-bad-pix" or ((exportSettings.fix_bad_pixels == 1) and " --bad-pix" or " --really-bad-pix"),
							(exportSettings.debug_bad_pixels == true) and " --black-bad-pix" or "",
							(exportSettings.use_fullres == 0) and " --no-fullres" or " --fullres",
							(exportSettings.use_alias_map == 0) and " --no-alias-map" or " --alias-map",
							(exportSettings.use_stripe_fix == 0) and " --no-stripe-fix" or " --stripe-fix",
							(exportSettings.compress == 1) and " --compress" or ((exportSettings.compress == 2) and "--compress-lossy" or ""))
						mlLog:debugf("arguments : %s", args)
						
						if MAC_ENV then
						
							-- cr2hdr exec path for Mac (& force language to english; seem to be unnecessary LANG=C LC_ALL=C LANGUAGE=en)
							command = string.format('exec "%s/bin/cr2hdr%s" "%s"', pluginPath, args, filePath)
							if exportSettings.saveOutput then
							
								command = string.format('%s > "%s"', command, output)
							
							end
							
							mlLog:debugf("command : %s", command)
							
						else

							command = string.format('start /D "%s\\bin" /B /WAIT /BELOWNORMAL cr2hdr.exe%s "%s"', pluginPath, args, filePath)
							if exportSettings.saveOutput then
							
								command = string.format('%s ^> "%s"', command, output)
							
							end
							
							command = string.format('"%s"', command)
							
							mlLog:debugf("command : %s", command)
							
						end
						
						result = LrTasks.execute(command)
						
						if result == 0 then

							if LrFileUtils.exists(filePathDNG) ~= false then
							
								local success, moveError = LrFileUtils.copy(filePathDNG, importDNG)
								if success then
								
									local catalog = LrApplication:activeCatalog()
									local importDualiso = catalog:withWriteAccessDo('Import from ML DualISO',
										function(context)
											catalog:addPhoto(importDNG, rendition.photo, 'above')
											local photo = catalog:findPhotoByPath(importDNG)
											photo:setRawMetadata('label', exportSettings.label)
											
											if exportSettings.addToCollection and next(originalCollection) ~= nil then
	
												for k, v in pairs(originalCollection) do
													v:addPhotos({ photo })
												end
												
											end
											
											--if exportSettings.moveToTrash then
											--	mlLog:debug("moveToTrash")
											--	LrFileUtils.moveToTrash(originalFilePath)
											--end
											
										end,
										{timeout = 30}
									)
									
									if importDualiso ~= "executed" then
									
										LrDialogs.message (LOC "$$$/ML/Import/ErrorTitle=~ There was a problem ~", LOC"$$$/ML/Import/Error=Unable to import the dual iso file!^n^nPlease manually synchronise your folder")
										
									end
								else
									LrDialogs.message (LOC "$$$/ML/Move/ErrorTitle=~ Problem to copy the file ~", LOC ("$$$/ML/Move/Error=Unable to copy the file^n^nfrom:^n^1^nto:^n^2^nError:^n^3", filePathDNG, importDNG, moveError))
								end
								
							else
							
								if not exportSettings.verbose then
								
									LrDialogs.message(LOC "$$$/ML/Publish/noDualIsoTitle=~ Doesn't look like interlaced ISO ~", LOC("$$$/ML/Publish/noDualIso=The image ^1 does not appear to be dual iso", originalName))
									
								end
								
							end
							
						else
							
							LrDialogs.message(LOC "$$$/ML/Publish/Error/Title=~ Error to export ~", LOC("$$$/ML/Publish/Error=Unable to export ^1 to cr2hdr!^r^rError #^2^r^rReport it to MagicLantern Forum^n(don't forget to indicate this error number)", originalName, result), "critical")
						
						-- end if cr2hdr
						end
						
					-- end if success	
					end
					
				else
				
					if not exportSettings.verbose then
					
						LrDialogs.message(LOC "$$$/ML/Publish/AlreadyConvertTitle=~ Already treated ~", LOC("$$$/ML/Publish/AlreadyConvert=The file ^1 has already converted to dual iso", originalName))
						
					end
					
				end
			-- rendition skip
			end
			
		else
		
			if not exportSettings.verbose then
			
				LrDialogs.message(LOC "$$$/ML/Publish/Error/NotRawTitle=Wrong format", LOC "$$$/ML/Publish/Error/NotRaw=Only RAW (CR2 & DNG) photo is allowed to export to cr2hdr")
				
			end
		
		-- end if not raw
		end
	
	-- end for
	end
	
	progressScope:done()
	
-- end function
end

function exportServiceProvider.sectionsForTopOfDialog(f, propertyTable)
	local widthText = 300
	local widthSpacer = 5
	local fillRadio = 0.2
	return {
		{
			title = LOC "$$$/ML/ExportManager/Title=Magic Lantern Dual ISO",
			f:row {
				f:picture {
					value =  _PLUGIN:resourceId("logo.png"),
				},
			},
			f:row {
				f:push_button {
					title = LOC "$$$/ML/ExportManager/MLTopic=Magic Lantern Dual ISO thread",
					action = function()
						LrHttp.openUrlInBrowser("http://www.magiclantern.fm/forum/index.php?topic=7139.0")
					end,					
				},
				f:push_button {
					title = LOC "$$$/ML/ExportManager/MLDocTech=Technical doc",
					action = function()
						LrHttp.openUrlInBrowser("http://acoutts.com/a1ex/dual_iso.pdf")
					end,					
				},
				f:spacer {
					height = 20
				},
				
			},
			f:row {
				f:static_text {
					title = LOC "$$$/ML/ExportManager/Description=This plugin allows to convert a Dual ISO picture with cr2hdr (thanks a1ex) from Magic Lantern.\n\nHow it works :\n\n- Export picture to temp folder\n- Add -dualiso suffix to picture\n- Convert picture with cr2hdr\n- Import converted picture into this LR Catalog\n- Add DualISO to the label\n- Add converted picture to the collection of the original",
				},
			},
		},
		{
			title = LOC "$$$/ML/ExportManager/CRTitle=cr2hdr optional arguments",
			visible = false,
			f:tab_view {
				value = "CRTab1",
				fill_horizontal = 1,
				f:tab_view_item {
					title = "Shortcuts",
					identifier = "CRTab1",
					f:row {
						f:checkbox {
							size = "small",
							text_color = LrColor("gray"),
							title = "--fast",
							tooltip = "disable most postprocessing steps (fast, but low quality)\n(--mean23, --no-cs, --no-fullres, --no-alias-map, --no-stripe-fix, --no-bad-pix)",
							value = bind 'shortcut_fast',
							checked_value = true,
							unchecked_value = false,
						},
					},
				},
				f:tab_view_item {
					title = "Interpolation methods",
					identifier = "CRTab2",
					f:row {
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--amaze-edge",
							tooltip = "use a temporary demosaic step (AMaZE) followed by edge-directed interpolation (default)",
							value = bind 'interp_method',
							checked_value = 0,
						},
					},
					f:row {
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--mean23",
							tooltip = "average the nearest 2 or 3 pixels of the same color from the Bayer grid (faster)",
							value = bind 'interp_method',
							checked_value = 1,
						},
					},
				},
				f:tab_view_item {
					title = "Chroma smoothing",
					identifier = "CRTab3",
					f:row {
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--no-cs",
							tooltip = "disable chroma smoothing",
							value = bind 'chroma_smooth_method',
							checked_value = 0,
						},
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--cs2x2",
							tooltip = "apply 2x2 chroma smoothing in noisy and aliased areas (default)",
							value = bind 'chroma_smooth_method',
							checked_value = 2,
						},
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--cs3x3",
							tooltip = "apply 3x3 chroma smoothing in noisy and aliased areas",
							value = bind 'chroma_smooth_method',
							checked_value = 3,
						},
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--cs5x5",
							tooltip = "apply 5x5 chroma smoothing in noisy and aliased areas",
							value = bind 'chroma_smooth_method',
							checked_value = 5,
						},
					},
				},
				f:tab_view_item {
					title = "Bad pixel handling",
					identifier = "CRTab4",
					f:row {
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--no-bad-pix",
							tooltip = "disable bad pixel fixing (try it if you shoot stars)",
							value = bind 'fix_bad_pixels',
							checked_value = 3,
						},
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--bad-pix",
							tooltip = "",
							value = bind 'fix_bad_pixels',
							checked_value = 1,
						},
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--really-bad-pix",
							tooltip = "aggressive bad pixel fix, at the expense of detail and aliasing",
							value = bind 'fix_bad_pixels',
							checked_value = 2,
						},
					},
					f:spacer {},
					f:row {
						f:checkbox {
							size = "small",
							text_color = LrColor("gray"),
							title = "--black-bad-pix",
							tooltip = "mark all bad pixels as black (for troubleshooting)",
							value = bind 'debug_bad_pixels',
							checked_value = true,
							unchecked_value = false,
						},
					},
				},
				f:tab_view_item {
					title = "Other postprocessing steps",
					identifier = "CRTab5",
					f:row {
						f:radio_button {
							fill_horizontal = fillRadio,
							size = "small",
							text_color = LrColor("gray"),
							title = "--no-fullres",
							tooltip = "disable full-resolution blending",
							value = bind 'use_fullres',
							checked_value = 0,
						},
						f:radio_button {
							fill_horizontal = fillRadio,
							size = "small",
							text_color = LrColor("gray"),
							title = "--fullres",
							tooltip = "",
							value = bind 'use_fullres',
							checked_value = 1,
						},
					},
					f:row {
						f:radio_button {
							fill_horizontal = fillRadio,
							size = "small",
							text_color = LrColor("gray"),
							title = "--no-alias-map",
							tooltip = "disable alias map, used to fix aliasing in deep shadows",
							value = bind 'use_alias_map',
							checked_value = 0,
						},
						f:radio_button {
							fill_horizontal = fillRadio,
							size = "small",
							text_color = LrColor("gray"),
							title = "--alias-map",
							tooltip = "",
							value = bind 'use_alias_map',
							checked_value = 1,
						},
					},
					f:row {
						f:radio_button {
							fill_horizontal = fillRadio,
							size = "small",
							text_color = LrColor("gray"),
							title = "--no-stripe-fix",
							tooltip = "disable horizontal stripe fix",
							value = bind 'use_stripe_fix',
							checked_value = 0,
						},
						f:radio_button {
							fill_horizontal = fillRadio,
							size = "small",
							text_color = LrColor("gray"),
							title = "--stripe-fix",
							tooltip = "",
							value = bind 'use_stripe_fix',
							checked_value = 1,
						},
					},
				},
				f:tab_view_item {
					title = "DNG compression",
					identifier = "CRTab6",
					f:row {
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--no-compress",
							tooltip = "Lossless DNG compression",
							value = bind 'compress',
							checked_value = 0,
						},
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--compress",
							tooltip = "Lossless DNG compression",
							value = bind 'compress',
							checked_value = 1,
						},
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--compress-lossy",
							tooltip = "Lossy DNG compression (be careful, may destroy shadow detail)",
							value = bind 'compress',
							checked_value = 2,
						},
					},
				},
			},
		},
		{
			title = LOC "$$$/ML/ExportManager/ASTitle=Advanced settings",
			visible = false,
			f:row {
				f:static_text {
					alignment = "right",
					width = widthText,
					title = LOC "$$$/ML/ExportManager/ASSuffix=Suffix for output dual iso file :",
				},
				f:spacer {
					width = widthSpacer,
				},
				f:static_text {
					size = "small",
					text_color = LrColor("gray"),
					title = "IMG_1234-",
				},
				f:edit_field {
					value = bind 'suffix',
					width = 60,
					
				},
				f:static_text {
					size = "small",
					text_color = LrColor("gray"),
					title = ".dng",
				},
			},
			f:row {
				f:static_text {
					width = widthText,
					alignment = "right",
					title = LOC "$$$/ML/ExportManager/ASLabel=Define the label for Dual ISO file :",
				},
				f:spacer {
					width = widthSpacer,
				},
				f:edit_field {
					value = bind 'label',
					width = 100,
					
				},
			},
			f:row {

				f:static_text {
					width = widthText,
					alignment = "right",
					title = LOC "$$$/ML/ExportManager/ASSubfolder=Add the converted file to this subfolder :",
				},
				f:spacer {
					width = widthSpacer,
				},
				f:edit_field {
					value = bind 'subfolder',
					width = 100,
					placeholder_string = LOC "$$$/ML/ExportManager/ASSubfolderPL=Parent",
				},
				f:static_text {
					text_color = LrColor("gray"),
					alignment = "right",
					title = LOC "$$$/ML/ExportManager/ASSubfolderInfo=leave blank to reimport into the same folder",
				},
			},
			f:row {
				f:static_text {
					width = widthText,
					alignment = "right",
					title = LOC "$$$/ML/ExportManager/ASCollection=Add Dual ISO file to collection ?",
				},
				f:spacer {
					width = widthSpacer,
				},
				f:checkbox {
					size = "small",
					text_color = LrColor("gray"),
					title = LOC "$$$/ML/ExportManager/ASCollectionInfo=Uncheck to not add to the original picture collection",
					value = bind 'addToCollection',
					checked_value = true,
					unchecked_value = false,
				},
			},
			f:row {
				f:static_text {
					width = widthText,
					alignment = "right",
					title = LOC "$$$/ML/ExportManager/ASLog=Keep log file after conversion ?",
				},
				f:spacer {
					width = widthSpacer,
				},
				f:checkbox {
					size = "small",
					text_color = LrColor("gray"),
					title = LOC "$$$/ML/ExportManager/ASLogInfo=Uncheck to remove",
					value = bind 'saveOutput',
					checked_value = true,
					unchecked_value = false,
				},
			},
			f:row {
				f:static_text {
					width = widthText,
					alignment = "right",
					title = LOC "$$$/ML/ExportManager/ASVerbose=Disable verbose mode ?",
				},
				f:spacer {
					width = widthSpacer,
				},
				f:checkbox {
					size = "small",
					text_color = LrColor("gray"),
					title = LOC "$$$/ML/ExportManager/ASVerboseInfo=Check to mute",
					value = bind 'verbose',
					checked_value = true,
					unchecked_value = false,
				},
			},
			--f:row {
			--	f:static_text {
			--		width = widthText,
			--		alignment = "right",
			--		title = LOC "$$$/ML/ExportManager/ASMoveToTrash=Move original to trash ?",
			--	},
			--	f:spacer {
			--		width = widthSpacer,
			--	},
			--	f:checkbox {
			--		size = "small",
			--		text_color = LrColor("red"),
			--		title = LOC "$$$/ML/ExportManager/ASMoveToTrashInfo=Check to remove (be carefull!)",
			--		value = bind 'moveToTrash',
			--		checked_value = true,
			--		unchecked_value = false,
			--	},
			--},
		},
	}
end

return exportServiceProvider